#include "binary_arithmetic_filter.hpp"
#include "workflow_graph.hpp"


namespace xios
{
  CScalarFieldArithmeticFilter::CScalarFieldArithmeticFilter(CGarbageCollector& gc, const std::string& op, double value)
    : CFilter(gc, 1, this)
    , op(operatorExpr.getOpScalarField(op))
    , value(value)
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CScalarFieldArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    bool building_graph = this->tag ? data[0]->timestamp >= this->start_graph && data[0]->timestamp <= this->end_graph : false;

    if(building_graph)
    {
      this->filterID = InvalidableObject::filterIdGenerator++;   
      int edgeID = InvalidableObject::edgeIdGenerator++; 

      if(CWorkflowGraph::mapFieldToFilters_ptr_with_info==0) CWorkflowGraph::mapFieldToFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_edge >;
      if(CWorkflowGraph::mapFilters_ptr_with_info==0) CWorkflowGraph::mapFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_node>;

      std::cout<<"CScalarFieldArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_name = "S "+op_expression +" F Filter" ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_class = 2 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_filled = 1 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].expected_entry_nb = 1 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].date = data[0]->date ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].timestamp = data[0]->timestamp ;

      if(CWorkflowGraph::build_begin)
      {
        CWorkflowGraph::addEdge(edgeID, this->filterID, data[0]);

        (*CWorkflowGraph::mapFilters_ptr_with_info)[data[0]->src_filterID].filter_filled = 0 ;
      }
      else CWorkflowGraph::build_begin = true;
    }

    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    packet->status = data[0]->status;
    if(building_graph) packet->src_filterID = this->filterID;
    packet->field = this->field;

    if (packet->status == CDataPacket::NO_ERROR)
      packet->data.reference(op(value, data[0]->data));

    return packet;
  }

  CFieldScalarArithmeticFilter::CFieldScalarArithmeticFilter(CGarbageCollector& gc, const std::string& op, double value)
    : CFilter(gc, 1, this)
    , op(operatorExpr.getOpFieldScalar(op))
    , value(value)
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "pow")
      op_expression = "^";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CFieldScalarArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    bool building_graph = this->tag ? data[0]->timestamp >= this->start_graph && data[0]->timestamp <= this->end_graph : false;

    if(building_graph)
    {
      this->filterID = InvalidableObject::filterIdGenerator++;
      int edgeID = InvalidableObject::edgeIdGenerator++;
 
      if(CWorkflowGraph::mapFieldToFilters_ptr_with_info==0) CWorkflowGraph::mapFieldToFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_edge >;
      if(CWorkflowGraph::mapFilters_ptr_with_info==0) CWorkflowGraph::mapFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_node>;

      std::cout<<"CFieldScalarArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_name = "F "+op_expression +" S Filter" ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_class = 2 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_filled = 1 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].expected_entry_nb = 1 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].date = data[0]->date ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].timestamp = data[0]->timestamp ;

      if(CWorkflowGraph::build_begin)
      {
        CWorkflowGraph::addEdge(edgeID, this->filterID, data[0]);

        (*CWorkflowGraph::mapFilters_ptr_with_info)[data[0]->src_filterID].filter_filled = 0 ;
      }
      else CWorkflowGraph::build_begin = true;
    }

    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    packet->status = data[0]->status;
    if(building_graph) packet->src_filterID = this->filterID;
    packet->field = this->field;
    

    if (packet->status == CDataPacket::NO_ERROR)
      packet->data.reference(op(data[0]->data, value));

    return packet;
  }

  CFieldFieldArithmeticFilter::CFieldFieldArithmeticFilter(CGarbageCollector& gc, const std::string& op)
    : CFilter(gc, 2, this)
    , op(operatorExpr.getOpFieldField(op))
  { 
    StdString input_op_expression=op;
    if(input_op_expression == "add")
      op_expression = "+";
    else if(input_op_expression == "minus")
      op_expression = "-";
    else if(input_op_expression == "mult")
      op_expression = "x";
    else if(input_op_expression == "div")
      op_expression = "/";
    else if(input_op_expression == "pow")
      op_expression = "^";
    else if(input_op_expression == "eq")
      op_expression = "=";
    else if(input_op_expression == "lt")
      op_expression = "<";
    else if(input_op_expression == "gt")
      op_expression = ">";
    else if(input_op_expression == "le")
      op_expression = "<=";
    else if(input_op_expression == "ge")
      op_expression = ">=";
    else if(input_op_expression == "ne")
      op_expression = "!=";
    else
      op_expression = " ";
  };

  CDataPacketPtr CFieldFieldArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    bool building_graph = this->tag ? data[0]->timestamp >= this->start_graph && data[0]->timestamp <= this->end_graph : false;

    if(building_graph)
    {

      this->filterID = InvalidableObject::filterIdGenerator++;
      int edgeID = InvalidableObject::edgeIdGenerator++;
   
      if(CWorkflowGraph::mapFieldToFilters_ptr_with_info==0) CWorkflowGraph::mapFieldToFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_edge >;
      if(CWorkflowGraph::mapFilters_ptr_with_info==0) CWorkflowGraph::mapFilters_ptr_with_info = new std::unordered_map <int, graph_info_box_node>;

      std::cout<<"CFieldFieldArithmeticFilter::apply filter tag = "<<this->tag<<std::endl;

      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_name = "F "+op_expression +" F Filter" ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_class = 2 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_filled = 1 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].expected_entry_nb = 2 ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].date = data[0]->date ;
      (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].timestamp = data[0]->timestamp ;

      if(CWorkflowGraph::build_begin)
      {

        CWorkflowGraph::addEdge(edgeID, this->filterID, data[0]);

        edgeID = InvalidableObject::edgeIdGenerator++;

        CWorkflowGraph::addEdge(edgeID, this->filterID, data[1]);

        (*CWorkflowGraph::mapFilters_ptr_with_info)[data[0]->src_filterID].filter_filled = 0 ;
        (*CWorkflowGraph::mapFilters_ptr_with_info)[data[1]->src_filterID].filter_filled = 0 ;
      }
      CWorkflowGraph::build_begin = true;
    }


    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    if(building_graph) packet->src_filterID = this->filterID;
    packet->field = this->field;
    

    if (data[0]->status != CDataPacket::NO_ERROR)
      packet->status = data[0]->status;
    else if (data[1]->status != CDataPacket::NO_ERROR)
      packet->status = data[1]->status;
    else
    {
      packet->status = CDataPacket::NO_ERROR;
      packet->data.reference(op(data[0]->data, data[1]->data));
    }

    return packet;
  }

  StdString CScalarFieldArithmeticFilter::GetName(void)    { return StdString("CScalarFieldArithmeticFilter"); }
  StdString CFieldScalarArithmeticFilter::GetName(void)    { return StdString("CFieldScalarArithmeticFilter"); }
  StdString CFieldFieldArithmeticFilter::GetName(void)     { return StdString("CFieldFieldArithmeticFilter"); }


} // namespace xios
