#include "graphviz.hpp"
#include "workflow_graph.hpp"

namespace xios
{

  CGraphviz::CGraphviz()
  { }

  /*!
   *
   */
  void CGraphviz::buildWorkflowGraphDot()
  TRY
  {
    if (CWorkflowGraph::mapFieldToFilters_ptr_with_info !=0 && !CWorkflowGraph::mapFieldToFilters_ptr_with_info->empty())
    {
      CWorkflowGraph::buildStaticWorkflow();
    
      typedef boost::property<boost::edge_name_t, std::string> EdgeProperty;
      typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::directedS, boost::no_property, EdgeProperty> Graph;

      // Input 1: nodes (=filters)
      vector<StdString>& nodes = CWorkflowGraph::filters;
      const int nbNodes = nodes.size();

      // Input 2: edges (=fields)
      vector<StdString>& edges = CWorkflowGraph::fields;

      // Input 3: nodes to edges connectivity
      vector<pair<int, int> >& nodesToEdges = CWorkflowGraph::fieldsToFilters;
      const int nbEdges = nodesToEdges.size();

      // Declare a graph object, adding the edges and edge properties
      Graph g(nbNodes);
      for (int j = 0; j < nbEdges; ++j)
      {
        graph_traits<Graph>::edge_descriptor e;
        bool inserted;
        boost::tie(e, inserted) = boost::add_edge(nodesToEdges[j].first, nodesToEdges[j].second, edges[j], g);
      }

      std::for_each(vertices(g).first, vertices(g).second, exercise_vertex<Graph>(g));

      std::map<std::string,std::string> graph_attr, vertex_attr, edge_attr;
      graph_attr["size"] = "5,5";
      graph_attr["rankdir"] = "LR";
      graph_attr["ratio"] = "fill";
      vertex_attr["shape"] = "record";
      vertex_attr["width"] = "2.2";
      vertex_attr["fontsize"] = "16";

      const std::string filename = "graph.dot";
      std::ofstream file (filename.c_str());

      boost::write_graphviz(file, g,
          boost::make_label_writer(&nodes[0]),
          boost::make_label_writer(get(edge_name, g)),
          boost::make_graph_attributes_writer(graph_attr, vertex_attr, edge_attr));
         
    }
  }
  CATCH




  void CGraphviz::buildWorkflowGraphVisjs_with_info()
  TRY
  {
    if (CWorkflowGraph::mapFilters_ptr_with_info !=0 && !CWorkflowGraph::mapFilters_ptr_with_info->empty())
    {
      CWorkflowGraph::buildStaticWorkflow_with_info();
    
      std::ofstream fs_json;
      fs_json.open ("graph_data.json", std::fstream::out);

      fs_json << "{ \"node\" : ["<<std::endl;
    
      for (auto it=CWorkflowGraph::mapFilters_ptr_with_info->begin(); it != CWorkflowGraph::mapFilters_ptr_with_info->end(); it++)
      {
        fs_json << "        {\"id\": "<<it->first +1<<"}, "<<std::endl;
        fs_json << "            {\"label\": \""<<it->second.filter_name<<"\"}, "<<std::endl  ;
        fs_json << "            {\"class\": "<<it->second.filter_class<<"}, "<<std::endl  ;
        fs_json << "            {\"filled\": "<<it->second.filter_filled<<"}, "<<std::endl  ;
        fs_json << "            {\"entry\": "<<it->second.expected_entry_nb<<"}, "<<std::endl  ;
        // fs_json << "            {\"date\": \""<<it->second.date<<"\"}, "<<std::endl  ;
        fs_json << "            {\"type\": \""<<it->second.transform_type<<"\"}, "<<std::endl  ;
      }
      fs_json << "    ]}"<<std::endl<<std::endl;

      fs_json << "{ \"edge\" : ["<<std::endl;

      for (auto it=CWorkflowGraph::mapFieldToFilters_ptr_with_info->begin(); it != CWorkflowGraph::mapFieldToFilters_ptr_with_info->end(); it++)
      {
        fs_json << "        {\"id\": "<<it->first +1<<"}, "<<std::endl;
        fs_json << "            {\"from\": "<<it->second.from +1<<"}, "<<std::endl;
        fs_json << "            {\"to\": "<<it->second.to+1<<"}, "<<std::endl  ;
        fs_json << "            {\"fid\": \""<<it->second.field_id<<"\"}, "<<std::endl  ;
        fs_json << "            {\"fname\": \""<<it->second.field_name<<"\"}, "<<std::endl  ;
        fs_json << "            {\"gid\": \""<<it->second.grid_id<<"\"}, "<<std::endl  ;
        fs_json << "            {\"date\": \""<<it->second.date<<"\"}, "<<std::endl  ;
      }
      fs_json << "    ]}"<<std::endl<<std::endl;


      

      

      fs_json.close();




      std::ofstream fs;
      fs.open ("graph_with_info.html", std::fstream::out);

      fs << "<html>" <<std::endl;
      fs << "<head>" <<std::endl;
      fs << "    <script type=\"text/javascript\" src=\"../../../vis-4.21.0/dist/vis.js\"></script>" <<std::endl;
      fs << "    <link href=\"../../../vis-4.21.0/dist/vis.css\" rel=\"stylesheet\" type=\"text/css\" />" <<std::endl <<std::endl;
      fs << "    <style type=\"text/css\">"<<std::endl;
      fs << "        #mynetwork {"<<std::endl;
      fs << "            width: 1000px;"<<std::endl;
      fs << "            height: 800px;"<<std::endl;
      fs << "            border: 1px solid lightgray"<<std::endl;
      fs << "        }"<<std::endl;
      fs << "    </style>"<<std::endl;
      fs << "</head>"<<std::endl;
      fs << "<body>"<<std::endl;
      fs << "<div id=\"mynetwork\"></div>"<<std::endl<<std::endl;
      fs << "<script type=\"text/javascript\">"<<std::endl;

      fs << "    var nodes = new vis.DataSet(["<<std::endl;
      for(int i=0; i<CWorkflowGraph::filters.size(); i++)
      {
        fs << "        {id: "<<i+1<<", label: \'"<<CWorkflowGraph::filters[i]<<"\'},"<<std::endl  ;
      }
      fs << "    ]);"<<std::endl<<std::endl;


      fs << "    var edges = new vis.DataSet(["<<std::endl;
      for(int i=0; i<CWorkflowGraph::fieldsToFilters.size(); i++)
      {
        fs <<"        {from: "<<CWorkflowGraph::fieldsToFilters[i].first+1<<", to: "<<CWorkflowGraph::fieldsToFilters[i].second+1<<", label: \'"<<CWorkflowGraph::fields[i]<<"\'},"<<std::endl;
      }
      fs << "    ]);"<<std::endl<<std::endl;

  
      fs << "    var container = document.getElementById(\'mynetwork\');" <<std::endl<<std::endl;
      fs << "    var data = {" <<std::endl;
      fs << "        nodes: nodes," <<std::endl;
      fs << "        edges: edges" <<std::endl;
      fs << "    };" <<std::endl;
      fs << "    var options = {"<<std::endl;
      fs << "        edges:{"<<std::endl;
      fs << "            smooth: false,"<<std::endl;
      fs << "            arrows: \'to\',"<<std::endl;
      fs << "            color: 'red',"<<std::endl;
      fs << "            font:{align: \'middle\'},"<<std::endl;
      fs << "        },"<<std::endl;
      fs << "        physics:{"<<std::endl;
      fs << "            enabled: false,"<<std::endl;
      fs << "        },"<<std::endl;
      fs << "};" <<std::endl<<std::endl;

      fs << "    var network = new vis.Network(container, data, options);" <<std::endl<<std::endl;

      fs << "</script>"<<std::endl;
      fs << "</body>"<<std::endl;
      fs << "</html>"<<std::endl;

      

      fs.close();
         
    }
  }
  CATCH



  void CGraphviz::showStaticWorkflowGraph()
  TRY
  {
    CWorkflowGraph::showStaticWorkflow();
  }
  CATCH
}
