#include "workflow_graph.hpp"

namespace xios
{

  std::unordered_map <int, graph_info_box_node> *CWorkflowGraph::mapFilters_ptr_with_info = 0;
  #pragma omp threadprivate(CWorkflowGraph::mapFilters_ptr_with_info)

  std::unordered_map <int, graph_info_box_edge > *CWorkflowGraph::mapFieldToFilters_ptr_with_info = 0;
  #pragma omp threadprivate(CWorkflowGraph::mapFieldToFilters_ptr_with_info)


  vector <StdString> CWorkflowGraph::filters;
  vector <StdString> CWorkflowGraph::fields;
  vector <pair<int, int> > CWorkflowGraph::fieldsToFilters;

  bool CWorkflowGraph::build_begin = false;
  #pragma omp threadprivate(CWorkflowGraph::build_begin)

  CWorkflowGraph::CWorkflowGraph()
  { }

  /*!
   * Reorganize information collected by XIOS for visualization.
   */
  void CWorkflowGraph::buildStaticWorkflow()
  TRY
  {
    
    if(mapFilters_ptr_with_info==0 || mapFieldToFilters_ptr_with_info==0) 
      std::cout<<"No graph information provided"<<std::endl;

    else // Create a list of filters (graph nodes)
    {
      size_t filterIdx = 0;
      filters.resize(mapFilters_ptr_with_info->size());
      fieldsToFilters.clear();
      fields.clear();
      for (auto it=mapFilters_ptr_with_info->begin(); it != mapFilters_ptr_with_info->end(); it++)
      {
        filters[it->first]=it->second.filter_name;
      }

      // Create a list of fields and their filter connectivity (graph edges and edge-to-node connectivity)
      for (auto it=mapFieldToFilters_ptr_with_info->begin(); it != mapFieldToFilters_ptr_with_info->end(); it++)
      {
        fieldsToFilters.push_back(make_pair(it->second.from,it->second.to));
        fields.push_back(it->second.field_id + " "+it->second.date.toString());
      }
    }
  }
  CATCH




/*!
   * Reorganize information collected by XIOS for visualization.
   */
  void CWorkflowGraph::buildStaticWorkflow_with_info()
  TRY
  {
    
    if(mapFilters_ptr_with_info==0 || mapFieldToFilters_ptr_with_info==0) 
      std::cout<<"No graph information provided"<<std::endl;

    else // Create a list of filters (graph nodes)
    {
      size_t filterIdx = 0;
      filters.resize(mapFilters_ptr_with_info->size());
      fieldsToFilters.clear();
      fields.clear();
      for (auto it=mapFilters_ptr_with_info->begin(); it != mapFilters_ptr_with_info->end(); it++)
      {
        filters[it->first]=it->second.filter_name;
      }

      // Create a list of fields and their filter connectivity (graph edges and edge-to-node connectivity)
      for (auto it=mapFieldToFilters_ptr_with_info->begin(); it != mapFieldToFilters_ptr_with_info->end(); it++)
      {
        fieldsToFilters.push_back(make_pair(it->second.from,it->second.to));
        fields.push_back(it->second.field_id + " "+it->second.date.toString());
      }
    }
  }
  CATCH



  void CWorkflowGraph::showStaticWorkflow()
  TRY
  {
    if(mapFilters_ptr_with_info!=0 && !mapFilters_ptr_with_info->empty())
    for (auto it=mapFilters_ptr_with_info->begin(); it != mapFilters_ptr_with_info->end(); it++)
    {
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].filter_name="<<it->second.filter_name<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].filter_class="<<it->second.filter_class<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].filter_fillled="<<it->second.filter_filled<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].expected_entry_nb="<<it->second.expected_entry_nb<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].date="<<it->second.date<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].timestamp="<<it->second.timestamp<<std::endl;
      std::cout<<"mapFilters_ptr_with_info["<<it->first<<"].transform_type="<<it->second.transform_type<<std::endl;
    }
    else std::cout<<"mapFilters_ptr_with_info empty"<<std::endl;

    if(mapFieldToFilters_ptr_with_info!=0 && !mapFieldToFilters_ptr_with_info->empty())
    for (auto it=mapFieldToFilters_ptr_with_info->begin(); it != mapFieldToFilters_ptr_with_info->end(); it++)
    {
      std::cout<<"mapFieldToFilters_ptr_with_info["<<it->first<<"].from = "<<it->second.from<<std::endl;
      std::cout<<"mapFieldToFilters_ptr_with_info["<<it->first<<"].to = "<<it->second.to<<std::endl;
      std::cout<<"mapFieldToFilters_ptr_with_info["<<it->first<<"].field_id = "<<it->second.field_id<<std::endl;
      std::cout<<"mapFieldToFilters_ptr_with_info["<<it->first<<"].field_name = "<<it->second.field_name<<std::endl;
      std::cout<<"mapFieldToFilters_ptr_with_info["<<it->first<<"].date = "<<it->second.date<<std::endl;
      
    }
    else std::cout<<"mapFieldToFilters_ptr_with_info empty"<<std::endl;


  }
  CATCH


  void CWorkflowGraph::addEdge(int edgeID, int toID, CDataPacketPtr packet)
  TRY
  {
    (*mapFieldToFilters_ptr_with_info)[edgeID].from = packet->src_filterID;
    (*mapFieldToFilters_ptr_with_info)[edgeID].to = toID;
    (*mapFieldToFilters_ptr_with_info)[edgeID].field_id = packet->field->getId();
    (*mapFieldToFilters_ptr_with_info)[edgeID].field_name = packet->field->name;
    (*mapFieldToFilters_ptr_with_info)[edgeID].grid_id = packet->field->grid->getId();
    (*mapFieldToFilters_ptr_with_info)[edgeID].date = packet->date;
    (*mapFieldToFilters_ptr_with_info)[edgeID].timestamp = packet->timestamp;
  }
  CATCH

}
